package FindApp::Object::State::Group::Behavior::Overloading;

use FindApp::Utils ":package";
use namespace::clean;

use parent PACKAGE->left (2) 
         + PACKAGE->right(2);

1;

__END__

=head1 NAME

FindApp::Object::State::Group::Behavior::Overloading -

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 METHODS

=head1 SEE ALSO

=head1 AUTHOR

=head1 LICENCE AND COPYRIGHT
