#!/usr/bin/env perl 

use t::setup;

my $Class = __TEST_PACKAGE__;
run_tests();

sub load_tests {
    require_ok $Class;
    {
        local $TODO = "need to fix import() sensitivity in $Class";
        use_ok $Class;
    }
}

__END__
